/**
 *  \file gtk_Stock.cpp
 */

#include "gtk_Stock.hpp"

#include <gtk/gtk.h>

/*#
   @beginmodule gtk
*/

namespace Falcon {
namespace Gtk {

/**
 *  \brief module init
 */
void Stock::modInit( Falcon::Module* mod )
{

    struct stockItem
    {
        const char* name;
        const char* id;
    };

    struct stockItem stockItems[] =
    {
    { "GTK_STOCK_ABOUT",            GTK_STOCK_ABOUT },
    { "GTK_STOCK_ADD",              GTK_STOCK_ADD },
    { "GTK_STOCK_APPLY",            GTK_STOCK_APPLY },
    { "GTK_STOCK_BOLD",             GTK_STOCK_BOLD },
    { "GTK_STOCK_CANCEL",           GTK_STOCK_CANCEL },
#if GTK_CHECK_VERSION( 2, 16, 0 )
    { "GTK_STOCK_CAPS_LOCK_WARNING",GTK_STOCK_CAPS_LOCK_WARNING },
#endif
    { "GTK_STOCK_CDROM",            GTK_STOCK_CDROM },
    { "GTK_STOCK_CLEAR",            GTK_STOCK_CLEAR },
    { "GTK_STOCK_CLOSE",            GTK_STOCK_CLOSE },
    { "GTK_STOCK_COLOR_PICKER",     GTK_STOCK_COLOR_PICKER },
    { "GTK_STOCK_CONVERT",          GTK_STOCK_CONVERT },
    { "GTK_STOCK_CONNECT",          GTK_STOCK_CONNECT },
    { "GTK_STOCK_COPY",             GTK_STOCK_COPY },
    { "GTK_STOCK_CUT",              GTK_STOCK_CUT },
    { "GTK_STOCK_DELETE",           GTK_STOCK_DELETE },
    { "GTK_STOCK_DIALOG_AUTHENTICATION",GTK_STOCK_DIALOG_AUTHENTICATION },
    { "GTK_STOCK_DIALOG_ERROR",     GTK_STOCK_DIALOG_ERROR },
    { "GTK_STOCK_DIALOG_INFO",      GTK_STOCK_DIALOG_INFO },
    { "GTK_STOCK_DIALOG_QUESTION",  GTK_STOCK_DIALOG_QUESTION },
    { "GTK_STOCK_DIALOG_WARNING",   GTK_STOCK_DIALOG_WARNING },
    { "GTK_STOCK_DIRECTORY",        GTK_STOCK_DIRECTORY },
    { "GTK_STOCK_DISCARD",          GTK_STOCK_DISCARD },
    { "GTK_STOCK_DISCONNECT",       GTK_STOCK_DISCONNECT },
    { "GTK_STOCK_DND",              GTK_STOCK_DND },
    { "GTK_STOCK_DND_MULTIPLE",     GTK_STOCK_DND_MULTIPLE },
    { "GTK_STOCK_EDIT",             GTK_STOCK_EDIT },
    { "GTK_STOCK_EXECUTE",          GTK_STOCK_EXECUTE },
    { "GTK_STOCK_FILE",             GTK_STOCK_FILE },
    { "GTK_STOCK_FIND",             GTK_STOCK_FIND },
    { "GTK_STOCK_FIND_AND_REPLACE", GTK_STOCK_FIND_AND_REPLACE },
    { "GTK_STOCK_FLOPPY",           GTK_STOCK_FLOPPY },
    { "GTK_STOCK_FULLSCREEN",       GTK_STOCK_FULLSCREEN },
    { "GTK_STOCK_GOTO_BOTTOM",      GTK_STOCK_GOTO_BOTTOM },
    { "GTK_STOCK_GOTO_FIRST",       GTK_STOCK_GOTO_FIRST },
    { "GTK_STOCK_GOTO_LAST",        GTK_STOCK_GOTO_LAST },
    { "GTK_STOCK_GOTO_TOP",         GTK_STOCK_GOTO_TOP },
    { "GTK_STOCK_GO_BACK",          GTK_STOCK_GO_BACK },
    { "GTK_STOCK_GO_DOWN",          GTK_STOCK_GO_DOWN },
    { "GTK_STOCK_GO_FORWARD",       GTK_STOCK_GO_FORWARD },
    { "GTK_STOCK_GO_UP",            GTK_STOCK_GO_UP },
    { "GTK_STOCK_HARDDISK",         GTK_STOCK_HARDDISK },
    { "GTK_STOCK_HELP",             GTK_STOCK_HELP },
    { "GTK_STOCK_HOME",             GTK_STOCK_HOME },
    { "GTK_STOCK_INDENT",           GTK_STOCK_INDENT },
    { "GTK_STOCK_INDEX",            GTK_STOCK_INDEX },
    { "GTK_STOCK_INFO",             GTK_STOCK_INFO },
    { "GTK_STOCK_ITALIC",           GTK_STOCK_ITALIC },
    { "GTK_STOCK_JUMP_TO",          GTK_STOCK_JUMP_TO },
    { "GTK_STOCK_JUSTIFY_CENTER",   GTK_STOCK_JUSTIFY_CENTER },
    { "GTK_STOCK_JUSTIFY_FILL",     GTK_STOCK_JUSTIFY_FILL },
    { "GTK_STOCK_JUSTIFY_LEFT",     GTK_STOCK_JUSTIFY_LEFT },
    { "GTK_STOCK_JUSTIFY_RIGHT",    GTK_STOCK_JUSTIFY_RIGHT },
    { "GTK_STOCK_LEAVE_FULLSCREEN", GTK_STOCK_LEAVE_FULLSCREEN },
    { "GTK_STOCK_MEDIA_FORWARD",    GTK_STOCK_MEDIA_FORWARD },
    { "GTK_STOCK_MEDIA_NEXT",       GTK_STOCK_MEDIA_NEXT },
    { "GTK_STOCK_MEDIA_PAUSE",      GTK_STOCK_MEDIA_PAUSE },
    { "GTK_STOCK_MEDIA_PLAY",       GTK_STOCK_MEDIA_PLAY },
    { "GTK_STOCK_MEDIA_PREVIOUS",   GTK_STOCK_MEDIA_PREVIOUS },
    { "GTK_STOCK_MEDIA_RECORD",     GTK_STOCK_MEDIA_RECORD },
    { "GTK_STOCK_MEDIA_REWIND",     GTK_STOCK_MEDIA_REWIND },
    { "GTK_STOCK_MEDIA_STOP",       GTK_STOCK_MEDIA_STOP },
    { "GTK_STOCK_MISSING_IMAGE",    GTK_STOCK_MISSING_IMAGE },
    { "GTK_STOCK_NETWORK",          GTK_STOCK_NETWORK },
    { "GTK_STOCK_NEW",              GTK_STOCK_NEW },
    { "GTK_STOCK_NO",               GTK_STOCK_NO },
    { "GTK_STOCK_OK",               GTK_STOCK_OK },
    { "GTK_STOCK_OPEN",             GTK_STOCK_OPEN },
    { "GTK_STOCK_ORIENTATION_LANDSCAPE",GTK_STOCK_ORIENTATION_LANDSCAPE },
    { "GTK_STOCK_ORIENTATION_PORTRAIT",GTK_STOCK_ORIENTATION_PORTRAIT },
    { "GTK_STOCK_ORIENTATION_REVERSE_LANDSCAPE",GTK_STOCK_ORIENTATION_REVERSE_LANDSCAPE },
    { "GTK_STOCK_ORIENTATION_REVERSE_PORTRAIT",GTK_STOCK_ORIENTATION_REVERSE_PORTRAIT },
#if GTK_CHECK_VERSION( 2, 14, 0 )
    { "GTK_STOCK_PAGE_SETUP",       GTK_STOCK_PAGE_SETUP },
#endif
    { "GTK_STOCK_PASTE",            GTK_STOCK_PASTE },
    { "GTK_STOCK_PREFERENCES",      GTK_STOCK_PREFERENCES },
    { "GTK_STOCK_PRINT",            GTK_STOCK_PRINT },
#if GTK_CHECK_VERSION( 2, 14, 0 )
    { "GTK_STOCK_PRINT_ERROR",      GTK_STOCK_PRINT_ERROR },
    { "GTK_STOCK_PRINT_PAUSED",     GTK_STOCK_PRINT_PAUSED },
#endif
    { "GTK_STOCK_PRINT_PREVIEW",    GTK_STOCK_PRINT_PREVIEW },
#if GTK_CHECK_VERSION( 2, 14, 0 )
    { "GTK_STOCK_PRINT_REPORT",     GTK_STOCK_PRINT_REPORT },
    { "GTK_STOCK_PRINT_WARNING",    GTK_STOCK_PRINT_WARNING },
#endif
    { "GTK_STOCK_PROPERTIES",       GTK_STOCK_PROPERTIES },
    { "GTK_STOCK_QUIT",             GTK_STOCK_QUIT },
    { "GTK_STOCK_REDO",             GTK_STOCK_REDO },
    { "GTK_STOCK_REFRESH",          GTK_STOCK_REFRESH },
    { "GTK_STOCK_REMOVE",           GTK_STOCK_REMOVE },
    { "GTK_STOCK_REVERT_TO_SAVED",  GTK_STOCK_REVERT_TO_SAVED },
    { "GTK_STOCK_SAVE",             GTK_STOCK_SAVE },
    { "GTK_STOCK_SAVE_AS",          GTK_STOCK_SAVE_AS },
    { "GTK_STOCK_SELECT_ALL",       GTK_STOCK_SELECT_ALL },
    { "GTK_STOCK_SELECT_COLOR",     GTK_STOCK_SELECT_COLOR },
    { "GTK_STOCK_SELECT_FONT",      GTK_STOCK_SELECT_FONT },
    { "GTK_STOCK_SORT_ASCENDING",   GTK_STOCK_SORT_ASCENDING },
    { "GTK_STOCK_SORT_DESCENDING",  GTK_STOCK_SORT_DESCENDING },
    { "GTK_STOCK_SPELL_CHECK",      GTK_STOCK_SPELL_CHECK },
    { "GTK_STOCK_STOP",             GTK_STOCK_STOP },
    { "GTK_STOCK_STRIKETHROUGH",    GTK_STOCK_STRIKETHROUGH },
    { "GTK_STOCK_UNDELETE",         GTK_STOCK_UNDELETE },
    { "GTK_STOCK_UNDERLINE",        GTK_STOCK_UNDERLINE },
    { "GTK_STOCK_UNDO",             GTK_STOCK_UNDO },
    { "GTK_STOCK_UNINDENT",         GTK_STOCK_UNINDENT },
    { "GTK_STOCK_YES",              GTK_STOCK_YES },
    { "GTK_STOCK_ZOOM_100",         GTK_STOCK_ZOOM_100 },
    { "GTK_STOCK_ZOOM_FIT",         GTK_STOCK_ZOOM_FIT },
    { "GTK_STOCK_ZOOM_IN",          GTK_STOCK_ZOOM_IN },
    { "GTK_STOCK_ZOOM_OUT",         GTK_STOCK_ZOOM_OUT },
    { NULL, NULL }
    };

    for ( struct stockItem* stock = stockItems; stock->name; ++stock )
        mod->addConstant( stock->name, stock->id, true );
}


} // Gtk
} // Falcon
